// Screen Space Radial Blur
//
// Radial Blur
// Pat 'Hawthorne' Shearon
// pat.shearon@gmail.com www.theregulators.org
// http://www.truevision3d.com/forums/printpage.html;topic=18745.0
//
// This version is provided under a Creative Commons Attribution 3.0 License
// http://creativecommons.org/licenses/by/3.0/ 



//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

float BlurWidth = -0.1;


//texture
texture Tex <string uiname="Texture";>;
sampler Texture = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;


struct VS_INPUT
{
   float4 position : POSITION;
   float2 texCoord : TEXCOORD;
};
struct VS_OUTPUT
{
   float4 position : POSITION;
   float2 texCoord : TEXCOORD;
};
#define PS_INPUT VS_OUTPUT

//Vertex Shader Output
VS_OUTPUT vs_main(const VS_INPUT IN)
{
   VS_OUTPUT OUT;

   OUT.position = mul(IN.position, tWVP);
   OUT.texCoord = mul(IN.texCoord, tTex);

   return OUT;
}

float4 ps_main(float2 TexCoord : TEXCOORD0) : COLOR0
{
    float2 Center   = {0.5, 0.5};
    float BlurStart = 1;
    float samples   = 16;


    float4 Color = 0;

    for(int i = 0; i < samples; i++)
    {
       float scale = BlurStart + BlurWidth * (i / (samples - 1));
       Color      += tex2D(Texture, (TexCoord - 0.5) * scale + Center );
    }
    Color /= samples;

    return Color;
}


//Technique Calls
technique PostProcess
{
   pass Pass_0
   {
      VertexShader = compile vs_3_0 vs_main();
      PixelShader = compile ps_3_0 ps_main();
   }
}
