// Screen Space Ripple Effect
//
// Rippl EFfect
// Pat 'Hawthorne' Shearon
// pat.shearon@gmail.com www.theregulators.org
// http://www.truevision3d.com/forums/printpage.html;topic=18745.0
//
// This version is provided under a Creative Commons Attribution 3.0 License
// http://creativecommons.org/licenses/by/3.0/ 


//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;


//floats
float Timer = 3.14159f / 0.75f;                // pi/.75 is a good default
float Distortion = 1;        // 1 is a good default
float2 Center = (0.5f,0.5f);        // 0.5,0.5 is the screen center



//texture
texture Tex <string uiname="Texture";>;
sampler Texture = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

struct VS_INPUT
{
   float4 position : POSITION;
   float2 texCoord : TEXCOORD;
};
struct VS_OUTPUT
{
   float4 position : POSITION;
   float2 texCoord : TEXCOORD;
};
#define PS_INPUT VS_OUTPUT

VS_OUTPUT vs_main(const VS_INPUT IN)
{
   VS_OUTPUT OUT;

   OUT.position = mul(IN.position, tWVP);
   OUT.texCoord = mul(IN.texCoord, tTex);

   return OUT;
}


float4 ps_main(float2 texCoord: TEXCOORD0) : COLOR0
{
    float2 distance = abs(texCoord - Center);
    float scalar = length(distance);

    // invert the scale so 1 is centerpoint
    scalar = abs(1 - scalar);
        
    // calculate how far to distort for this pixel    
    float sinoffset = sin(Timer / scalar);
    sinoffset = clamp(sinoffset, 0, 1);
    
    // calculate which direction to distort
    float sinsign = cos(Timer / scalar);
    
    // reduce the distortion effect
    sinoffset = sinoffset * Distortion/32;
    
    // pick a pixel on the screen for this pixel, based on
    // the calculated offset and direction
    float4 color = tex2D(Texture, texCoord+(sinoffset*sinsign));    
            
    return color;
}

technique PostProcess 
{ 
   pass Pass_0 
   { 
      VertexShader = compile vs_1_1 vs_main(); 
      PixelShader = compile ps_2_0 ps_main(); 
   } 
}
